@@ -8,7 +8,9 @@ use libc::{close, dup, dup2, pipe, STDERR_FILENO, STDOUT_FILENO};
 use rand::prelude::SliceRandom;
 use rand::Rng;
 use similar::TextDiff;
+use std::env::temp_dir;
 use std::ffi::OsString;
+use std::fs::File;
 use std::io::{Seek, SeekFrom, Write};
 use std::os::fd::{AsRawFd, RawFd};
 use std::process::{Command, Stdio};
@@ -231,6 +233,10 @@ pub fn run_gnu_cmd(
         command.arg(arg);
     }
 
+    // See https://github.com/uutils/coreutils/issues/6794
+    // uutils' coreutils is not locale-aware, and aims to mirror/be compatible with GNU Core Utilities's LC_ALL=C behavior
+    command.env("LC_ALL", "C");
+
     let output = if let Some(input_str) = pipe_input {
         // We have an pipe input
         command
@@ -374,7 +380,7 @@ pub fn generate_random_string(max_length: usize) -> String {
     let invalid_utf8 = [0xC3, 0x28]; // Invalid UTF-8 sequence
     let mut result = String::new();
 
-    for _ in 0..rng.gen_range(1..=max_length) {
+    for _ in 0..rng.gen_range(0..=max_length) {
         if rng.gen_bool(0.9) {
             let ch = valid_utf8.choose(&mut rng).unwrap();
             result.push(*ch);
@@ -388,3 +394,23 @@ pub fn generate_random_string(max_length: usize) -> String {
 
     result
 }
+
+pub fn generate_random_file() -> Result<String, std::io::Error> {
+    let mut rng = rand::thread_rng();
+    let file_name: String = (0..10)
+        .map(|_| rng.gen_range(b'a'..=b'z') as char)
+        .collect();
+    let mut file_path = temp_dir();
+    file_path.push(file_name);
+
+    let mut file = File::create(&file_path)?;
+
+    let content_length = rng.gen_range(10..1000);
+    let content: String = (0..content_length)
+        .map(|_| (rng.gen_range(b' '..=b'~') as char))
+        .collect();
+
+    file.write_all(content.as_bytes())?;
+
+    Ok(file_path.to_str().unwrap().to_string())
+}
