@@ -13,7 +13,7 @@ use uucore::error::UResult;
 use uucore::fs::FileInformation;
 
 #[cfg(unix)]
-use std::os::unix::io::AsRawFd;
+use std::os::fd::{AsFd, AsRawFd};
 
 /// Linux splice support
 #[cfg(any(target_os = "linux", target_os = "android"))]
@@ -125,12 +125,12 @@ struct OutputState {
 }
 
 #[cfg(unix)]
-trait FdReadable: Read + AsRawFd {}
+trait FdReadable: Read + AsFd + AsRawFd {}
 #[cfg(not(unix))]
 trait FdReadable: Read {}
 
 #[cfg(unix)]
-impl<T> FdReadable for T where T: Read + AsRawFd {}
+impl<T> FdReadable for T where T: Read + AsFd + AsRawFd {}
 #[cfg(not(unix))]
 impl<T> FdReadable for T where T: Read {}
 
@@ -467,6 +467,13 @@ fn write_fast<R: FdReadable>(handle: &mut InputHandle<R>) -> CatResult<()> {
         }
         stdout_lock.write_all(&buf[..n])?;
     }
+
+    // If the splice() call failed and there has been some data written to
+    // stdout via while loop above AND there will be second splice() call
+    // that will succeed, data pushed through splice will be output before
+    // the data buffered in stdout.lock. Therefore additional explicit flush
+    // is required here.
+    stdout_lock.flush()?;
     Ok(())
 }
 
@@ -540,9 +547,16 @@ fn write_new_line<W: Write>(
     state: &mut OutputState,
     is_interactive: bool,
 ) -> CatResult<()> {
-    if state.skipped_carriage_return && options.show_ends {
-        writer.write_all(b"^M")?;
+    if state.skipped_carriage_return {
+        if options.show_ends {
+            writer.write_all(b"^M")?;
+        } else {
+            writer.write_all(b"\r")?;
+        }
         state.skipped_carriage_return = false;
+
+        write_end_of_line(writer, options.end_of_line().as_bytes(), is_interactive)?;
+        return Ok(());
     }
     if !state.at_line_start || !options.squeeze_blank || !state.one_blank_kept {
         state.one_blank_kept = true;
@@ -550,10 +564,7 @@ fn write_new_line<W: Write>(
             write!(writer, "{0:6}\t", state.line_number)?;
             state.line_number += 1;
         }
-        writer.write_all(options.end_of_line().as_bytes())?;
-        if is_interactive {
-            writer.flush()?;
-        }
+        write_end_of_line(writer, options.end_of_line().as_bytes(), is_interactive)?;
     }
     Ok(())
 }
@@ -623,10 +634,10 @@ fn write_nonprint_to_end<W: Write>(in_buf: &[u8], writer: &mut W, tab: &[u8]) ->
             9 => writer.write_all(tab),
             0..=8 | 10..=31 => writer.write_all(&[b'^', byte + 64]),
             32..=126 => writer.write_all(&[byte]),
-            127 => writer.write_all(&[b'^', b'?']),
+            127 => writer.write_all(b"^?"),
             128..=159 => writer.write_all(&[b'M', b'-', b'^', byte - 64]),
             160..=254 => writer.write_all(&[b'M', b'-', byte - 128]),
-            _ => writer.write_all(&[b'M', b'-', b'^', b'?']),
+            _ => writer.write_all(b"M-^?"),
         }
         .unwrap();
         count += 1;
