@@ -10,6 +10,7 @@ use crate::common::util::TestScenario;
 use crate::common::util::UChild;
 #[cfg(unix)]
 use nix::sys::signal::Signal;
+#[cfg(feature = "echo")]
 use regex::Regex;
 use std::env;
 use std::path::Path;
@@ -35,13 +36,14 @@ impl Target {
         Self { child }
     }
     fn send_signal(&mut self, signal: Signal) {
-        Command::new("kill")
+        let _ = Command::new("kill")
             .args(&[
                 format!("-{}", signal as i32),
                 format!("{}", self.child.id()),
             ])
             .spawn()
-            .expect("failed to send signal");
+            .expect("failed to send signal")
+            .wait();
         self.child.delay(100);
     }
     fn is_alive(&mut self) -> bool {
@@ -78,6 +80,15 @@ fn test_env_version() {
         .stdout_contains(util_name!());
 }
 
+#[test]
+fn test_env_permissions() {
+    new_ucmd!()
+        .arg(".")
+        .fails()
+        .code_is(126)
+        .stderr_is("env: '.': Permission denied\n");
+}
+
 #[test]
 fn test_echo() {
     #[cfg(target_os = "windows")]
@@ -98,6 +109,7 @@ fn test_if_windows_batch_files_can_be_executed() {
     assert!(result.stdout_str().contains("Hello Windows World!"));
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug_1() {
     let ts = TestScenario::new(util_name!());
@@ -118,6 +130,7 @@ fn test_debug_1() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug_2() {
     let ts = TestScenario::new(util_name!());
@@ -144,6 +157,7 @@ fn test_debug_2() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug1_part_of_string_arg() {
     let ts = TestScenario::new(util_name!());
@@ -165,6 +179,7 @@ fn test_debug1_part_of_string_arg() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug2_part_of_string_arg() {
     let ts = TestScenario::new(util_name!());
@@ -651,7 +666,7 @@ fn test_env_with_empty_executable_double_quotes() {
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "dirname", feature = "echo"))]
 fn test_env_overwrite_arg0() {
     let ts = TestScenario::new(util_name!());
 
@@ -675,7 +690,7 @@ fn test_env_overwrite_arg0() {
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "echo"))]
 fn test_env_arg_argv0_overwrite() {
     let ts = TestScenario::new(util_name!());
 
@@ -723,7 +738,7 @@ fn test_env_arg_argv0_overwrite() {
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "echo"))]
 fn test_env_arg_argv0_overwrite_mixed_with_string_args() {
     let ts = TestScenario::new(util_name!());
 
@@ -876,6 +891,29 @@ fn test_env_arg_ignore_signal_empty() {
         .no_stderr()
         .stdout_contains("hello");
 }
+
+#[test]
+fn disallow_equals_sign_on_short_unset_option() {
+    let ts = TestScenario::new(util_name!());
+
+    ts.ucmd()
+        .arg("-u=")
+        .fails()
+        .code_is(125)
+        .stderr_contains("env: cannot unset '=': Invalid argument");
+    ts.ucmd()
+        .arg("-u=A1B2C3")
+        .fails()
+        .code_is(125)
+        .stderr_contains("env: cannot unset '=A1B2C3': Invalid argument");
+    ts.ucmd().arg("--split-string=A1B=2C3=").succeeds();
+    ts.ucmd()
+        .arg("--unset=")
+        .fails()
+        .code_is(125)
+        .stderr_contains("env: cannot unset '': Invalid argument");
+}
+
 #[cfg(test)]
 mod tests_split_iterator {
 
