@@ -6,14 +6,24 @@
 // spell-checker:ignore mydir
 use crate::common::util::TestScenario;
 use filetime::FileTime;
-use std::thread::sleep;
-use std::time::Duration;
+use rstest::rstest;
+use std::io::Write;
 
 #[test]
-fn test_invalid_arg() {
+fn test_mv_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_mv_missing_dest() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir = "dir";
+
+    at.mkdir(dir);
+
+    ucmd.arg(dir).fails();
+}
+
 #[test]
 fn test_mv_rename_dir() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -27,16 +37,6 @@ fn test_mv_rename_dir() {
     assert!(at.dir_exists(dir2));
 }
 
-#[test]
-fn test_mv_fail() {
-    let (at, mut ucmd) = at_and_ucmd!();
-    let dir1 = "test_mv_rename_dir";
-
-    at.mkdir(dir1);
-
-    ucmd.arg(dir1).fails();
-}
-
 #[test]
 fn test_mv_rename_file() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -133,7 +133,7 @@ fn test_mv_move_file_between_dirs() {
 
     assert!(at.file_exists(format!("{dir1}/{file}")));
 
-    ucmd.arg(&format!("{dir1}/{file}"))
+    ucmd.arg(format!("{dir1}/{file}"))
         .arg(dir2)
         .succeeds()
         .no_stderr();
@@ -299,9 +299,9 @@ fn test_mv_interactive_no_clobber_force_last_arg_wins() {
 
     scene
         .ucmd()
-        .args(&[file_a, file_b, "-f", "-i", "-n"])
-        .fails()
-        .stderr_is(format!("mv: not replacing '{file_b}'\n"));
+        .args(&[file_a, file_b, "-f", "-i", "-n", "--debug"])
+        .succeeds()
+        .stdout_contains("skipped 'b.txt'");
 
     scene
         .ucmd()
@@ -352,9 +352,9 @@ fn test_mv_no_clobber() {
     ucmd.arg("-n")
         .arg(file_a)
         .arg(file_b)
-        .fails()
-        .code_is(1)
-        .stderr_only(format!("mv: not replacing '{file_b}'\n"));
+        .arg("--debug")
+        .succeeds()
+        .stdout_contains("skipped 'test_mv_no_clobber_file_b");
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
@@ -468,7 +468,31 @@ fn test_mv_same_symlink() {
         .arg(file_c)
         .arg(file_a)
         .fails()
-        .stderr_is(format!("mv: '{file_c}' and '{file_a}' are the same file\n",));
+        .stderr_is(format!("mv: '{file_c}' and '{file_a}' are the same file\n"));
+}
+
+#[test]
+#[cfg(all(unix, not(target_os = "android")))]
+fn test_mv_same_broken_symlink() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.symlink_file("missing-target", "broken");
+
+    ucmd.arg("broken")
+        .arg("broken")
+        .fails()
+        .stderr_is("mv: 'broken' and 'broken' are the same file\n");
+}
+
+#[test]
+#[cfg(all(unix, not(target_os = "android")))]
+fn test_mv_symlink_into_target() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.mkdir("dir");
+    at.symlink_file("dir", "dir-link");
+
+    ucmd.arg("dir-link").arg("dir").succeeds();
 }
 
 #[test]
@@ -572,6 +596,30 @@ fn test_mv_simple_backup() {
     assert!(at.file_exists(format!("{file_b}~")));
 }
 
+#[test]
+fn test_mv_simple_backup_for_directory() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir_a = "test_mv_simple_backup_dir_a";
+    let dir_b = "test_mv_simple_backup_dir_b";
+
+    at.mkdir(dir_a);
+    at.mkdir(dir_b);
+    at.touch(format!("{dir_a}/file_a"));
+    at.touch(format!("{dir_b}/file_b"));
+    ucmd.arg("-T")
+        .arg("-b")
+        .arg(dir_a)
+        .arg(dir_b)
+        .succeeds()
+        .no_stderr();
+
+    assert!(!at.dir_exists(dir_a));
+    assert!(at.dir_exists(dir_b));
+    assert!(at.dir_exists(&format!("{dir_b}~")));
+    assert!(at.file_exists(format!("{dir_b}/file_a")));
+    assert!(at.file_exists(format!("{dir_b}~/file_b")));
+}
+
 #[test]
 fn test_mv_simple_backup_with_file_extension() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -863,14 +911,16 @@ fn test_mv_backup_off() {
 }
 
 #[test]
-fn test_mv_backup_no_clobber_conflicting_options() {
-    new_ucmd!()
-        .arg("--backup")
-        .arg("--no-clobber")
-        .arg("file1")
-        .arg("file2")
-        .fails()
-        .usage_error("options --backup and --no-clobber are mutually exclusive");
+fn test_mv_backup_conflicting_options() {
+    for conflicting_opt in ["--no-clobber", "--update=none-fail", "--update=none"] {
+        new_ucmd!()
+            .arg("--backup")
+            .arg(conflicting_opt)
+            .arg("file1")
+            .arg("file2")
+            .fails()
+            .usage_error("cannot combine --backup with -n/--no-clobber or --update=none-fail");
+    }
 }
 
 #[test]
@@ -905,6 +955,20 @@ fn test_mv_update_option() {
     assert!(!at.file_exists(file_b));
 }
 
+#[test]
+fn test_mv_update_with_dest_ending_with_slash() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let source = "source";
+    let dest = "destination/";
+
+    at.mkdir("source");
+
+    ucmd.arg("--update").arg(source).arg(dest).succeeds();
+
+    assert!(!at.dir_exists(source));
+    assert!(at.dir_exists(dest));
+}
+
 #[test]
 fn test_mv_arg_update_none() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -958,9 +1022,9 @@ fn test_mv_arg_update_older_dest_not_older() {
     let old_content = "file1 content\n";
     let new_content = "file2 content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -985,9 +1049,9 @@ fn test_mv_arg_update_none_then_all() {
     let old_content = "old content\n";
     let new_content = "new content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -1013,9 +1077,9 @@ fn test_mv_arg_update_all_then_none() {
     let old_content = "old content\n";
     let new_content = "new content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -1039,9 +1103,9 @@ fn test_mv_arg_update_older_dest_older() {
     let old_content = "file1 content\n";
     let new_content = "file2 content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -1065,9 +1129,9 @@ fn test_mv_arg_update_short_overwrite() {
     let old_content = "file1 content\n";
     let new_content = "file2 content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -1091,9 +1155,9 @@ fn test_mv_arg_update_short_no_overwrite() {
     let old_content = "file1 content\n";
     let new_content = "file2 content\n";
 
-    at.write(old, old_content);
-
-    sleep(Duration::from_secs(1));
+    let mut f = at.make_file(old);
+    f.write_all(old_content.as_bytes()).unwrap();
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
 
     at.write(new, new_content);
 
@@ -1158,6 +1222,32 @@ fn test_mv_overwrite_dir() {
     assert!(at.dir_exists(dir_b));
 }
 
+#[test]
+fn test_mv_no_target_dir_with_dest_not_existing() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir_a = "a";
+    let dir_b = "b";
+
+    at.mkdir(dir_a);
+    ucmd.arg("-T").arg(dir_a).arg(dir_b).succeeds().no_output();
+
+    assert!(!at.dir_exists(dir_a));
+    assert!(at.dir_exists(dir_b));
+}
+
+#[test]
+fn test_mv_no_target_dir_with_dest_not_existing_and_ending_with_slash() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir_a = "a";
+    let dir_b = "b/";
+
+    at.mkdir(dir_a);
+    ucmd.arg("-T").arg(dir_a).arg(dir_b).succeeds().no_output();
+
+    assert!(!at.dir_exists(dir_a));
+    assert!(at.dir_exists(dir_b));
+}
+
 #[test]
 fn test_mv_overwrite_nonempty_dir() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -1324,24 +1414,6 @@ fn test_mv_interactive_error() {
         .is_empty());
 }
 
-#[test]
-fn test_mv_into_self() {
-    let scene = TestScenario::new(util_name!());
-    let at = &scene.fixtures;
-    let dir1 = "dir1";
-    let dir2 = "dir2";
-    at.mkdir(dir1);
-    at.mkdir(dir2);
-
-    scene
-        .ucmd()
-        .arg(dir1)
-        .arg(dir2)
-        .arg(dir2)
-        .fails()
-        .stderr_contains("mv: cannot move 'dir2' to a subdirectory of itself, 'dir2/dir2'");
-}
-
 #[test]
 fn test_mv_arg_interactive_skipped() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -1360,10 +1432,9 @@ fn test_mv_arg_interactive_skipped_vin() {
     let (at, mut ucmd) = at_and_ucmd!();
     at.touch("a");
     at.touch("b");
-    ucmd.args(&["-vin", "a", "b"])
-        .fails()
-        .stderr_is("mv: not replacing 'b'\n")
-        .no_stdout();
+    ucmd.args(&["-vin", "a", "b", "--debug"])
+        .succeeds()
+        .stdout_contains("skipped 'b'");
 }
 
 #[test]
@@ -1392,27 +1463,46 @@ fn test_mv_into_self_data() {
     assert!(!at.file_exists(file1));
 }
 
-#[test]
-fn test_mv_directory_into_subdirectory_of_itself_fails() {
+#[rstest]
+#[case(vec!["mydir"], vec!["mydir", "mydir"], "mv: cannot move 'mydir' to a subdirectory of itself, 'mydir/mydir'")]
+#[case(vec!["mydir"], vec!["mydir/", "mydir/"], "mv: cannot move 'mydir/' to a subdirectory of itself, 'mydir/mydir'")]
+#[case(vec!["mydir"], vec!["./mydir", "mydir", "mydir/"], "mv: cannot move './mydir' to a subdirectory of itself, 'mydir/mydir'")]
+#[case(vec!["mydir"], vec!["mydir/", "mydir/mydir_2/"], "mv: cannot move 'mydir/' to a subdirectory of itself, 'mydir/mydir_2/'")]
+#[case(vec!["mydir/mydir_2"], vec!["mydir", "mydir/mydir_2"], "mv: cannot move 'mydir' to a subdirectory of itself, 'mydir/mydir_2/mydir'\n")]
+#[case(vec!["mydir/mydir_2"], vec!["mydir/", "mydir/mydir_2/"], "mv: cannot move 'mydir/' to a subdirectory of itself, 'mydir/mydir_2/mydir'\n")]
+#[case(vec!["mydir", "mydir_2"], vec!["mydir/", "mydir_2/", "mydir_2/"], "mv: cannot move 'mydir_2/' to a subdirectory of itself, 'mydir_2/mydir_2'")]
+#[case(vec!["mydir"], vec!["mydir/", "mydir"], "mv: cannot move 'mydir/' to a subdirectory of itself, 'mydir/mydir'")]
+#[case(vec!["mydir"], vec!["-T", "mydir", "mydir"], "mv: 'mydir' and 'mydir' are the same file")]
+#[case(vec!["mydir"], vec!["mydir/", "mydir/../"], "mv: 'mydir/' and 'mydir/../mydir' are the same file")]
+fn test_mv_directory_self(
+    #[case] dirs: Vec<&str>,
+    #[case] args: Vec<&str>,
+    #[case] expected_error: &str,
+) {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    let dir1 = "mydir";
-    let dir2 = "mydir/mydir_2";
-    at.mkdir(dir1);
-    at.mkdir(dir2);
-    scene.ucmd().arg(dir1).arg(dir2).fails().stderr_contains(
-        "mv: cannot move 'mydir' to a subdirectory of itself, 'mydir/mydir_2/mydir'",
-    );
-
-    // check that it also errors out with /
+    for dir in dirs {
+        at.mkdir_all(dir);
+    }
     scene
         .ucmd()
-        .arg(format!("{}/", dir1))
-        .arg(dir2)
+        .args(&args)
         .fails()
-        .stderr_contains(
-            "mv: cannot move 'mydir/' to a subdirectory of itself, 'mydir/mydir_2/mydir/'",
-        );
+        .stderr_contains(expected_error);
+}
+
+#[test]
+fn test_mv_dir_into_dir_with_source_name_a_prefix_of_target_name() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let source = "test";
+    let target = "test2";
+
+    at.mkdir(source);
+    at.mkdir(target);
+
+    ucmd.arg(source).arg(target).succeeds().no_output();
+
+    assert!(at.dir_exists(&format!("{target}/{source}")));
 }
 
 #[test]
@@ -1429,6 +1519,65 @@ fn test_mv_file_into_dir_where_both_are_files() {
         .stderr_contains("mv: failed to access 'b/': Not a directory");
 }
 
+#[test]
+fn test_mv_seen_file() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("a");
+    at.mkdir("b");
+    at.mkdir("c");
+    at.write("a/f", "a");
+    at.write("b/f", "b");
+
+    let result = ts.ucmd().arg("a/f").arg("b/f").arg("c").fails();
+
+    #[cfg(not(unix))]
+    assert!(result
+        .stderr_str()
+        .contains("will not overwrite just-created 'c\\f' with 'b/f'"));
+    #[cfg(unix)]
+    assert!(result
+        .stderr_str()
+        .contains("will not overwrite just-created 'c/f' with 'b/f'"));
+
+    // a/f has been moved into c/f
+    assert!(at.plus("c").join("f").exists());
+    // b/f still exists
+    assert!(at.plus("b").join("f").exists());
+    // a/f no longer exists
+    assert!(!at.plus("a").join("f").exists());
+}
+
+#[test]
+fn test_mv_seen_multiple_files_to_directory() {
+    let ts = TestScenario::new(util_name!());
+    let at = &ts.fixtures;
+
+    at.mkdir("a");
+    at.mkdir("b");
+    at.mkdir("c");
+    at.write("a/f", "a");
+    at.write("b/f", "b");
+    at.write("b/g", "g");
+
+    let result = ts.ucmd().arg("a/f").arg("b/f").arg("b/g").arg("c").fails();
+    #[cfg(not(unix))]
+    assert!(result
+        .stderr_str()
+        .contains("will not overwrite just-created 'c\\f' with 'b/f'"));
+    #[cfg(unix)]
+    assert!(result
+        .stderr_str()
+        .contains("will not overwrite just-created 'c/f' with 'b/f'"));
+
+    assert!(!at.plus("a").join("f").exists());
+    assert!(at.plus("b").join("f").exists());
+    assert!(!at.plus("b").join("g").exists());
+    assert!(at.plus("c").join("f").exists());
+    assert!(at.plus("c").join("g").exists());
+}
+
 #[test]
 fn test_mv_dir_into_file_where_both_are_files() {
     let scene = TestScenario::new(util_name!());
@@ -1443,6 +1592,60 @@ fn test_mv_dir_into_file_where_both_are_files() {
         .stderr_contains("mv: cannot stat 'a/': Not a directory");
 }
 
+#[test]
+fn test_mv_dir_into_path_slash() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    at.mkdir("a");
+    scene.ucmd().arg("a").arg("e/").succeeds();
+    assert!(at.dir_exists("e"));
+    at.mkdir("b");
+    at.mkdir("f");
+    scene.ucmd().arg("b").arg("f/").succeeds();
+    assert!(at.dir_exists("f/b"));
+}
+
+#[cfg(all(unix, not(any(target_os = "macos", target_os = "openbsd"))))]
+#[test]
+fn test_acl() {
+    use std::process::Command;
+
+    use crate::common::util::compare_xattrs;
+
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let path1 = "a";
+    let path2 = "b";
+    let file = "a/file";
+    let file_target = "b/file";
+    at.mkdir(path1);
+    at.mkdir(path2);
+    at.touch(file);
+
+    let path = at.plus_as_string(file);
+    // calling the command directly. xattr requires some dev packages to be installed
+    // and it adds a complex dependency just for a test
+    match Command::new("setfacl")
+        .args(["-m", "group::rwx", path1])
+        .status()
+        .map(|status| status.code())
+    {
+        Ok(Some(0)) => {}
+        Ok(_) => {
+            println!("test skipped: setfacl failed");
+            return;
+        }
+        Err(e) => {
+            println!("test skipped: setfacl failed with {e}");
+            return;
+        }
+    }
+
+    scene.ucmd().arg(&path).arg(path2).succeeds();
+
+    assert!(compare_xattrs(&file, &file_target));
+}
+
 // Todo:
 
 // $ at.touch a b
@@ -1455,3 +1658,112 @@ fn test_mv_dir_into_file_where_both_are_files() {
 // $ mv -v a b
 // mv: try to overwrite 'b', overriding mode 0444 (r--r--r--)? y
 // 'a' -> 'b'
+
+#[cfg(target_os = "linux")]
+mod inter_partition_copying {
+    use crate::common::util::TestScenario;
+    use std::fs::{read_to_string, set_permissions, write};
+    use std::os::unix::fs::{symlink, PermissionsExt};
+    use tempfile::TempDir;
+
+    // Ensure that the copying code used in an inter-partition move unlinks the destination symlink.
+    #[test]
+    pub(crate) fn test_mv_unlinks_dest_symlink() {
+        let scene = TestScenario::new(util_name!());
+        let at = &scene.fixtures;
+
+        // create a file in the current partition.
+        at.write("src", "src contents");
+
+        // create a folder in another partition.
+        let other_fs_tempdir =
+            TempDir::new_in("/dev/shm/").expect("Unable to create temp directory");
+
+        // create a file inside that folder.
+        let other_fs_file_path = other_fs_tempdir.path().join("other_fs_file");
+        write(&other_fs_file_path, "other fs file contents")
+            .expect("Unable to write to other_fs_file");
+
+        // create a symlink to the file inside the same directory.
+        let symlink_path = other_fs_tempdir.path().join("symlink_to_file");
+        symlink(&other_fs_file_path, &symlink_path).expect("Unable to create symlink_to_file");
+
+        // mv src to symlink in another partition
+        scene
+            .ucmd()
+            .arg("src")
+            .arg(symlink_path.to_str().unwrap())
+            .succeeds();
+
+        // make sure that src got removed.
+        assert!(!at.file_exists("src"));
+
+        // make sure symlink got unlinked
+        assert!(!symlink_path.is_symlink());
+
+        // make sure that file contents in other_fs_file didn't change.
+        assert_eq!(
+            read_to_string(&other_fs_file_path,).expect("Unable to read other_fs_file"),
+            "other fs file contents"
+        );
+
+        // make sure that src file contents got copied into new file created in symlink_path
+        assert_eq!(
+            read_to_string(&symlink_path).expect("Unable to read other_fs_file"),
+            "src contents"
+        );
+    }
+
+    // In an inter-partition move if unlinking the destination symlink fails, ensure
+    // that it would output the proper error message.
+    #[test]
+    pub(crate) fn test_mv_unlinks_dest_symlink_error_message() {
+        let scene = TestScenario::new(util_name!());
+        let at = &scene.fixtures;
+
+        // create a file in the current partition.
+        at.write("src", "src contents");
+
+        // create a folder in another partition.
+        let other_fs_tempdir =
+            TempDir::new_in("/dev/shm/").expect("Unable to create temp directory");
+
+        // create a file inside that folder.
+        let other_fs_file_path = other_fs_tempdir.path().join("other_fs_file");
+        write(&other_fs_file_path, "other fs file contents")
+            .expect("Unable to write to other_fs_file");
+
+        // create a symlink to the file inside the same directory.
+        let symlink_path = other_fs_tempdir.path().join("symlink_to_file");
+        symlink(&other_fs_file_path, &symlink_path).expect("Unable to create symlink_to_file");
+
+        // disable write for the target folder so that when mv tries to remove the
+        // the destination symlink inside the target directory it would fail.
+        set_permissions(other_fs_tempdir.path(), PermissionsExt::from_mode(0o555))
+            .expect("Unable to set permissions for temp directory");
+
+        // mv src to symlink in another partition
+        scene
+            .ucmd()
+            .arg("src")
+            .arg(symlink_path.to_str().unwrap())
+            .fails()
+            .stderr_contains("inter-device move failed:")
+            .stderr_contains("Permission denied");
+    }
+}
+
+#[test]
+fn test_mv_error_msg_with_multiple_sources_that_does_not_exist() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    at.mkdir("d");
+    scene
+        .ucmd()
+        .arg("a")
+        .arg("b/")
+        .arg("d")
+        .fails()
+        .stderr_contains("mv: cannot stat 'a': No such file or directory")
+        .stderr_contains("mv: cannot stat 'b/': No such file or directory");
+}
