@@ -106,7 +106,7 @@ fn test_very_large_range() {
         .collect();
     assert_eq!(result_seq.len(), num_samples, "Miscounted output length!");
     assert!(
-        result_seq.iter().all(|x| (0..=1234567890).contains(x)),
+        result_seq.iter().all(|x| (0..=1_234_567_890).contains(x)),
         "Output includes element not from range: {}",
         result.stdout_str()
     );
@@ -132,7 +132,7 @@ fn test_very_large_range_offset() {
     assert!(
         result_seq
             .iter()
-            .all(|x| (1234567890..=2147483647).contains(x)),
+            .all(|x| (1_234_567_890..=2_147_483_647).contains(x)),
         "Output includes element not from range: {}",
         result.stdout_str()
     );
@@ -143,7 +143,7 @@ fn test_range_repeat_no_overflow_1_max() {
     let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-rn1")
-        .arg(&format!("-i1-{upper_bound}"))
+        .arg(format!("-i1-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -161,7 +161,7 @@ fn test_range_repeat_no_overflow_0_max_minus_1() {
     let upper_bound = usize::MAX - 1;
     let result = new_ucmd!()
         .arg("-rn1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -179,7 +179,7 @@ fn test_range_permute_no_overflow_1_max() {
     let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i1-{upper_bound}"))
+        .arg(format!("-i1-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -197,7 +197,7 @@ fn test_range_permute_no_overflow_0_max_minus_1() {
     let upper_bound = usize::MAX - 1;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -218,7 +218,7 @@ fn test_range_permute_no_overflow_0_max() {
     let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -234,7 +234,13 @@ fn test_range_permute_no_overflow_0_max() {
 #[test]
 fn test_very_high_range_full() {
     let input_seq = vec![
-        2147483641, 2147483642, 2147483643, 2147483644, 2147483645, 2147483646, 2147483647,
+        2_147_483_641,
+        2_147_483_642,
+        2_147_483_643,
+        2_147_483_644,
+        2_147_483_645,
+        2_147_483_646,
+        2_147_483_647,
     ];
     let result = new_ucmd!().arg("-i2147483641-2147483647").succeeds();
     result.no_stderr();
@@ -320,7 +326,7 @@ fn test_echo_multi() {
         .stdout_str()
         .split('\n')
         .filter(|x| !x.is_empty())
-        .map(|x| x.into())
+        .map(std::convert::Into::into)
         .collect();
     result_seq.sort_unstable();
     assert_eq!(result_seq, ["a", "b", "c"], "Output is not a permutation");
@@ -335,7 +341,7 @@ fn test_echo_postfix() {
         .stdout_str()
         .split('\n')
         .filter(|x| !x.is_empty())
-        .map(|x| x.into())
+        .map(std::convert::Into::into)
         .collect();
     result_seq.sort_unstable();
     assert_eq!(result_seq, ["a", "b", "c"], "Output is not a permutation");
