@@ -13,6 +13,23 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_invalid_input() {
+    new_ucmd!()
+        .args(&["1", "1", "<", "."])
+        .fails()
+        .code_is(1)
+        .stderr_contains("tr: extra operand '<'");
+    #[cfg(unix)]
+    new_ucmd!()
+        .args(&["1", "1"])
+        // will test "tr 1 1 < ."
+        .set_stdin(std::process::Stdio::from(std::fs::File::open(".").unwrap()))
+        .fails()
+        .code_is(1)
+        .stderr_contains("tr: read error: Is a directory");
+}
+
 #[test]
 fn test_to_upper() {
     new_ucmd!()
@@ -1494,9 +1511,7 @@ fn test_multibyte_octal_sequence() {
         .args(&["-d", r"\501"])
         .pipe_in("(1Ł)")
         .succeeds()
-        // TODO
-        // A warning needs to be printed here
-        // See https://github.com/uutils/coreutils/issues/6821
+        .stderr_is("tr: warning: the ambiguous octal escape \\501 is being\n        interpreted as the 2-byte sequence \\050, 1\n")
         .stdout_is("Ł)");
 }
 
